@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' project.
- * Copyright (c) 2013-2021, Marek Gagolewski <https://www.gagolewski.com>
+ * Copyright (c) 2013-2023, Marek Gagolewski <https://www.gagolewski.com/>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -89,6 +89,7 @@ StriContainerUTF16::StriContainerUTF16(SEXP rstr, R_len_t _nrecycle, bool _shall
     if (this->n == 0)
         return; /* nothing more to do */
 
+    STRI_ASSERT(this->n > 0);
     this->str = new UnicodeString[this->n];
     STRI_ASSERT(this->str);
     if (!this->str) throw StriException(MSG__MEM_ALLOC_ERROR_WITH_SIZE,
@@ -96,11 +97,11 @@ StriContainerUTF16::StriContainerUTF16(SEXP rstr, R_len_t _nrecycle, bool _shall
     for (R_len_t i=0; i<this->n; ++i)
         this->str[i].setToBogus(); // in case it fails during conversion (this is NA)
 
-    /* Important: ICU provides full internationalization functionality
+    /* Important: ICU provides full internationalisation functionality
     without any conversion table data. The common library contains
     code to handle several important encodings algorithmically: US-ASCII,
     ISO-8859-1, UTF-7/8/16/32, SCSU, BOCU-1, CESU-8, and IMAP-mailbox-name */
-    StriUcnv ucnvASCII("US-ASCII");
+    //StriUcnv ucnvASCII("US-ASCII");
 #if defined(_WIN32) || defined(_WIN64)
     // #270: latin-1 is windows-1252 on Windows
     StriUcnv ucnvLatin1("WINDOWS-1252");
@@ -115,37 +116,41 @@ StriContainerUTF16::StriContainerUTF16(SEXP rstr, R_len_t _nrecycle, bool _shall
             continue; // keep NA
         }
 
-        if (IS_ASCII(curs)) {
-            // Version 1:
-            UConverter* ucnv = ucnvASCII.getConverter();
-            UErrorCode status = U_ZERO_ERROR;
-            this->str[i].setTo(
-                UnicodeString((const char*)CHAR(curs), (int32_t)LENGTH(curs), ucnv, status)
-            );
-            STRI__CHECKICUSTATUS_THROW(status, {/* do nothing special on err */})
-
-            // Performance improvement attempt #1:
-            // this->str[i] = new UnicodeString(UnicodeString::fromUTF8(CHAR(curs)));
-            // if (!this->str) throw StriException(MSG__MEM_ALLOC_ERROR);
-            // slower than the above
-
-            // Performance improvement attempt #2:
-            // Create UChar buf with LENGTH(curs) items, fill it with (CHAR(curs)[i], 0x00), i=1,...
-            // This wasn't faster than the ucnvASCII approach.
-
-            // Performance improvement attempt #3:
-            // slightly slower than ucnvASCII
-            // R_len_t curs_n = LENGTH(curs);
-            // const char* curs_s = CHAR(curs);
-            // this->str[i].remove(); // unset bogus (NA)
-            // UChar* buf = this->str[i].getBuffer(curs_n);
-            // for (R_len_t k=0; k<curs_n; ++k)
-            //   buf[k] = (UChar)curs_s[k]; // well, this is ASCII :)
-            // this->str[i].releaseBuffer(curs_n);
-        }
-        else if (IS_UTF8(curs)) {
+        // if (IS_ASCII(curs)) {
+        //     // Version 1:
+        //     UConverter* ucnv = ucnvASCII.getConverter();
+        //     UErrorCode status = U_ZERO_ERROR;
+        //     this->str[i].setTo(
+        //         UnicodeString((const char*)CHAR(curs), (int32_t)LENGTH(curs), ucnv, status)
+        //     );
+        //     STRI__CHECKICUSTATUS_THROW(status, {/* do nothing special on err */})
+        //
+        //     // Performance improvement attempt #1:
+        //     // this->str[i] = new UnicodeString(UnicodeString::fromUTF8(CHAR(curs)));
+        //     // if (!this->str) throw StriException(MSG__MEM_ALLOC_ERROR);
+        //     // slower than the above
+        //
+        //     // Performance improvement attempt #2:
+        //     // Create UChar buf with LENGTH(curs) items, fill it with (CHAR(curs)[i], 0x00), i=1,...
+        //     // This wasn't faster than the ucnvASCII approach.
+        //
+        //     // Performance improvement attempt #3:
+        //     // slightly slower than ucnvASCII
+        //     // R_len_t curs_n = LENGTH(curs);
+        //     // const char* curs_s = CHAR(curs);
+        //     // this->str[i].remove(); // unset bogus (NA)
+        //     // UChar* buf = this->str[i].getBuffer(curs_n);
+        //     // for (R_len_t k=0; k<curs_n; ++k)
+        //     //   buf[k] = (UChar)curs_s[k]; // well, this is ASCII :)
+        //     // this->str[i].releaseBuffer(curs_n);
+        // }
+        // else
+        if (IS_ASCII(curs) || IS_UTF8(curs)) {
             // using ucnvUTF8 is slower for UTF-8
             // the same is done for native encoding && ucnvNative_isUTF8
+
+            // this is slower if IS_ASCII than ucnvASCII, but doesn't limit
+            // the input string length to 858993458 characters (#487)
             this->str[i].setTo(UnicodeString::fromUTF8(CHAR(curs)));
         }
         else if (IS_LATIN1(curs)) {
@@ -329,11 +334,14 @@ SEXP StriContainerUTF16::toR(R_len_t i) const
  * @param adj2 adjust for \code{i2}
  *
  * @version 0.5-1 (Marek Gagolewski, 2014-12-21)
- *    #132 incorrect behavior for i2[j2] == i2[j2+1]
+ *    #132 incorrect behaviour for i2[j2] == i2[j2+1]
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-29) ignore NA and negative indexes
  */
-void StriContainerUTF16::UChar16_to_UChar32_index(R_len_t i,
-        int* i1, int* i2, const int ni, int adj1, int adj2)
-{
+void StriContainerUTF16::UChar16_to_UChar32_index(
+    R_len_t i,
+    int* i1, int* i2, const int ni, int adj1, int adj2
+) {
     const UnicodeString* str_data = &(this->get(i));
     const UChar* cstr = str_data->getBuffer();
     const int nstr = str_data->length();
@@ -344,9 +352,11 @@ void StriContainerUTF16::UChar16_to_UChar32_index(R_len_t i,
     int i16 = 0;
     int i32 = 0;
     while (i16 < nstr && (j1 < ni || j2 < ni)) {
+
         while (j1 < ni && i1[j1] <= i16) {
+            if (i1[j1] == NA_INTEGER || i1[j1] < 0) { ++j1; continue; }
 #ifndef NDEBUG
-            if (j1 < ni-1 && i1[j1] > i1[j1+1])
+            if (j1 < ni-1 && i1[j1+1] != NA_INTEGER && i1[j1+1] >= 0 && i1[j1] > i1[j1+1])
                 throw StriException("DEBUG: stri__UChar16_to_UChar32_index 1");
 #endif
             i1[j1] = i32 + adj1;
@@ -354,8 +364,9 @@ void StriContainerUTF16::UChar16_to_UChar32_index(R_len_t i,
         }
 
         while (j2 < ni && i2[j2] <= i16) {
+            if (i2[j2] == NA_INTEGER || i2[j2] < 0) { ++j2; continue; }
 #ifndef NDEBUG
-            if (j2 < ni-1 && i2[j2] > i2[j2+1])
+            if (j2 < ni-1 && i2[j2+1] != NA_INTEGER && i2[j2+1] >= 0 && i2[j2] > i2[j2+1])
                 throw StriException("DEBUG: stri__UChar16_to_UChar32_index 2");
 #endif
             i2[j2] = i32 + adj2;
@@ -369,6 +380,7 @@ void StriContainerUTF16::UChar16_to_UChar32_index(R_len_t i,
 
     // CONVERT LAST:
     while (j1 < ni && i1[j1] <= nstr) {
+        if (i1[j1] == NA_INTEGER || i1[j1] < 0) { ++j1; continue; }
 //#ifndef NDEBUG
 //      if (j1 < ni-1 && i1[j1] >= i1[j1+1])
 //         throw StriException("DEBUG: stri__UChar16_to_UChar32_index 3");
@@ -378,6 +390,7 @@ void StriContainerUTF16::UChar16_to_UChar32_index(R_len_t i,
     }
 
     while (j2 < ni && i2[j2] <= nstr) {
+        if (i2[j2] == NA_INTEGER || i2[j2] < 0) { ++j2; continue; }
 //#ifndef NDEBUG
 //      if (j2 < ni-1 && i2[j2] >= i2[j2+1])
 //         throw StriException("DEBUG: stri__UChar16_to_UChar32_index 4");
