@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' project.
- * Copyright (c) 2013-2021, Marek Gagolewski <https://www.gagolewski.com>
+ * Copyright (c) 2013-2023, Marek Gagolewski <https://www.gagolewski.com/>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -243,6 +243,9 @@
 #define MSG__ARG_EXPECTED_NOT_EMPTY \
    "argument `%s` should be a non-empty vector"
 
+#define MSG__ARG_EXPECTED_NOT_NULL \
+   "argument `%s` should not be a NULL"
+
 #define MSG__ARG_EXPECTED_1_STRING \
    "argument `%s` should be a single character string; only the first element is used"
 
@@ -357,5 +360,7 @@
 #define MSG__U_CHARSET_IS_UTF8 \
    "system ICU assumes that the default character set is always UTF-8, and hence this function has no effect"
 
+#define MSG__CHARSXP_2147483647 \
+    "Elements of character vectors (CHARSXPs) are limited to 2^31-1 bytes"
 
 #endif
